<?php

namespace Drupal\Tests\readmore_extrafield\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Tests the readmore_extrafield javascript functionalities.
 *
 * @group readmore_extrafield
 */
class ReadMoreExtraFieldFunctionalJavascriptTest extends WebDriverTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'node',
    'field',
    'field_ui',
    'test_page_test',
    'extra_field',
    'extra_field_plus',
    'readmore_extrafield',
    'layout_discovery',
    'layout_builder',
  ];

  /**
   * A user with admin permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $adminUser;

  /**
   * A user with authenticated permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $user;

  /**
   * A node.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected $node;


  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->createContentType(['type' => 'article']);
    $this->node = $this->drupalCreateNode([
      'title' => 'VerySpecificTestTitle',
      'type' => 'article',
      'body' => 'Body field value.',
    ]);

    $this->config('system.site')->set('page.front', '/test-page')->save();

    $this->user = $this->drupalCreateUser([]);
    $this->adminUser = $this->drupalCreateUser([]);
    $this->adminUser->addRole($this->createAdminRole('admin', 'admin'));
    $this->adminUser->save();
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Test setting apply if setted via ui.
   */
  public function testSetFieldSettingsViaUi() {
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface $display_repository */
    $display_repository = \Drupal::service('entity_display.repository');
    $display_repository->getViewDisplay('node', 'article')
      ->setComponent('extra_field_readmore_extrafield', [
        'type' => 'extra_field_readmore_extrafield',
        'settings' => [
          'link_title' => 'Link Title Test',
          'link_classes' => 'test-link-class',
          'link_attr_title' => 'Link Attr Title',
          'link_attr_rel' => 'alternate',
          'link_attr_target' => '_self',
        ],
      ])
      ->enable()
      ->save();
    $this->drupalGet('/admin/structure/types/manage/article/display');
    $page->pressButton('edit-fields-extra-field-readmore-extrafield-settings-edit');
    $session->waitForElementVisible('css', 'input[id*="edit-fields-extra-field-readmore-extrafield-settings-edit-form"]');
    $this->submitForm([
      'fields[extra_field_readmore_extrafield][settings_edit_form][settings][link_title]' => 'Link Title Test Overwritten',
      'fields[extra_field_readmore_extrafield][settings_edit_form][settings][link_classes]' => 'test-link-class-overwritten',
      'fields[extra_field_readmore_extrafield][settings_edit_form][settings][link_attr_title]' => 'Link Attr Title Overwritten',
      'fields[extra_field_readmore_extrafield][settings_edit_form][settings][link_attr_rel]' => 'author',
      'fields[extra_field_readmore_extrafield][settings_edit_form][settings][link_attr_target]' => '_top',
    ], 'Update');
    $page->pressButton('edit-submit');
    $this->drupalGet('/node/' . $this->node->id());

    // Check all settings are set on the Link:
    $session->elementExists('css', 'a.test-link-class-overwritten.readmore-extrafield-link');
    $session->elementExists('css', 'div.readmore-extrafield > a.test-link-class-overwritten.readmore-extrafield-link');
    $session->elementTextEquals('css', 'a.test-link-class-overwritten.readmore-extrafield-link', 'Link Title Test Overwritten');
    $session->elementAttributeContains('css', 'a.test-link-class-overwritten.readmore-extrafield-link', 'title', 'Link Attr Title Overwritten');
    $session->elementAttributeContains('css', 'a.test-link-class-overwritten.readmore-extrafield-link', 'rel', 'author');
    $session->elementAttributeContains('css', 'a.test-link-class-overwritten.readmore-extrafield-link', 'target', '_top');
  }

  /**
   * Tests the field settings with all options empty.
   */
  public function testFieldOptionsEmptyFallback() {
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface $display_repository */
    $display_repository = \Drupal::service('entity_display.repository');
    $display_repository->getViewDisplay('node', 'article')
      ->setComponent('extra_field_readmore_extrafield', [
        'type' => 'extra_field_readmore_extrafield',
        'settings' => [
          'link_title' => '',
          'link_classes' => '',
          'link_attr_title' => '',
          'link_attr_rel' => '',
          'link_attr_target' => '',
        ],
      ])
      ->enable()
      ->save();

    $this->drupalGet('/node/' . $this->node->id());

    // Check all settings are (not) set on the Link:
    $session->elementExists('css', 'a.readmore-extrafield-link');
    $session->elementExists('css', 'div.readmore-extrafield > a.readmore-extrafield-link');
    $session->elementTextEquals('css', 'a.readmore-extrafield-link', 'Read more');
    $session->elementAttributeNotExists('css', 'a.readmore-extrafield-link', 'title');
    $session->elementAttributeNotExists('css', 'a.readmore-extrafield-link', 'rel');
    $session->elementAttributeNotExists('css', 'a.readmore-extrafield-link', 'target');
  }

  /**
   * Tests to see if the field behaves correctly inside layout builder.
   *
   * @todo Finish this test and change values.
   */
  public function todoTestFieldInLayoutBuilder() {
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    $driver = $this->getSession()->getDriver();
    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface $display_repository */
    $display_repository = \Drupal::service('entity_display.repository');
    $display_repository->getViewDisplay('node', 'article')
      ->setComponent('extra_field_readmore_extrafield', [
        'type' => 'extra_field_readmore_extrafield',
        'settings' => [
          'link_title' => 'Link Title Test',
          'link_classes' => 'test-link-class',
          'link_attr_title' => 'Link Attr Title',
          'link_attr_rel' => 'alternate',
          'link_attr_target' => '_self',
        ],
      ])
      ->enable()
      ->save();
    // Set the field inside layout builder:
    $this->drupalGet('/admin/structure/types/manage/article/display');
    $page->checkField('edit-layout-enabled');
    $page->pressButton('edit-submit');
    $this->drupalGet('/admin/structure/types/manage/article/display/default/layout');
    $session->pageTextContainsOnce('Placeholder for the "Read more" field');
    $page->pressButton('Add block');
    $session->waitForElementVisible('css', 'div#drupal-off-canvas');
    $page->pressButton('Read more');
    $session->waitForElementVisible('css', 'div[id*="edit-settings-admin-label"]');
    $session->elementExists('css', 'input[id*="edit-settings-label"]');
    // @todo Check if other settings exist (link_to_entity and wrapper).
    // @todo Set settings and label.
    $page->pressButton('Add block');
    $page->pressButton('edit-submit');
    $this->drupalGet('/node/' . $this->node->id());
    $session->elementTextEquals('css', 'body > div > div > main > div > div > article > div > p', 'VerySpecificTestTitle (from extra_field_plus example)');
  }

}
